"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryBuilder = exports.QueryBuilderErrors = exports.ArrayQueryOperatorEnum = exports.LogicalQueryOperatorEnum = exports.EvaluationQueryOperatorEnum = exports.ComparisonQueryOperatorEnum = void 0;
var utils_1 = require("../utils");
var ComparisonQueryOperatorEnum;
(function (ComparisonQueryOperatorEnum) {
    ComparisonQueryOperatorEnum["GREATER_THAN_EQUALS"] = "$gte";
    ComparisonQueryOperatorEnum["GREATER_THAN"] = "$gt";
    ComparisonQueryOperatorEnum["EQUALS"] = "$eq";
    ComparisonQueryOperatorEnum["IN"] = "$in";
    ComparisonQueryOperatorEnum["LESS_THAN"] = "$lt";
    ComparisonQueryOperatorEnum["LESS_THAN_EQUALS"] = "$lte";
    ComparisonQueryOperatorEnum["NOT_EQUALS"] = "$ne";
    ComparisonQueryOperatorEnum["NOT_IN"] = "$nin";
})(ComparisonQueryOperatorEnum = exports.ComparisonQueryOperatorEnum || (exports.ComparisonQueryOperatorEnum = {}));
var EvaluationQueryOperatorEnum;
(function (EvaluationQueryOperatorEnum) {
    EvaluationQueryOperatorEnum["REGEX"] = "$regex";
})(EvaluationQueryOperatorEnum = exports.EvaluationQueryOperatorEnum || (exports.EvaluationQueryOperatorEnum = {}));
var LogicalQueryOperatorEnum;
(function (LogicalQueryOperatorEnum) {
    LogicalQueryOperatorEnum["AND"] = "$and";
    LogicalQueryOperatorEnum["NOT"] = "$not";
    LogicalQueryOperatorEnum["NOR"] = "$nor";
    LogicalQueryOperatorEnum["OR"] = "$or";
})(LogicalQueryOperatorEnum = exports.LogicalQueryOperatorEnum || (exports.LogicalQueryOperatorEnum = {}));
var ArrayQueryOperatorEnum;
(function (ArrayQueryOperatorEnum) {
    ArrayQueryOperatorEnum["ALL"] = "$all";
})(ArrayQueryOperatorEnum = exports.ArrayQueryOperatorEnum || (exports.ArrayQueryOperatorEnum = {}));
var QueryBuilderErrors;
(function (QueryBuilderErrors) {
    QueryBuilderErrors["MISSING_OPERATOR"] = "You have to specify an operator";
    QueryBuilderErrors["UNKNOWN_OPERATOR"] = "Unknown operator passed";
    QueryBuilderErrors["MISSING_FIELD"] = "The filter query requires a field";
    QueryBuilderErrors["MISSING_VALUE"] = "The filter query requires a value";
    QueryBuilderErrors["MISSING_FILTERS"] = "No filters property was specified";
    QueryBuilderErrors["NOT_A_NUMBER"] = "This filter requires a number as value";
    QueryBuilderErrors["NOT_AN_ARRAY"] = "This filter requires an array as value";
    QueryBuilderErrors["NOT_A_STRING"] = "This filter requires a strinig as value";
    QueryBuilderErrors["INVALID_REGEX"] = "This is not a valid regular expression";
})(QueryBuilderErrors = exports.QueryBuilderErrors || (exports.QueryBuilderErrors = {}));
var QueryBuilder = /** @class */ (function () {
    function QueryBuilder(logger) {
        this.logger = logger;
    }
    QueryBuilder.prototype.buildAll = function (filters) {
        return filters.map(this.build.bind(this)).filter(Boolean);
    };
    QueryBuilder.prototype.build = function (filter) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        var _this = this;
        // throw an error if no operator is specified
        this.logger.debug('[QueryBuilder.build]: Received Filter', filter);
        if (filter.operator == null)
            throw new Error(QueryBuilderErrors.MISSING_OPERATOR);
        switch (filter.operator) {
            case LogicalQueryOperatorEnum.AND:
            case LogicalQueryOperatorEnum.NOR:
            case LogicalQueryOperatorEnum.OR:
                // throw an error if no filters are specified
                if (filter.filters == null)
                    throw new Error(QueryBuilderErrors.MISSING_FILTERS);
                var childFilters = filter.filters
                    .map(function (child) { return _this.build(child); })
                    .filter(Boolean);
                // we do not want to add a logical filter with no children
                if (childFilters.length === 0)
                    return null;
                // we do not need a logical query for only one child --> return the child-query instead
                if (childFilters.length === 1)
                    return childFilters[0];
                return _a = {},
                    _a[filter.operator] = childFilters,
                    _a;
            case LogicalQueryOperatorEnum.NOT:
                if (!filter.field)
                    throw new Error(QueryBuilderErrors.MISSING_FIELD);
                var mappedFilter = this.build(__assign(__assign({}, filter.filter), { field: filter.field }));
                return mappedFilter
                    ? (_b = {},
                        _b[LogicalQueryOperatorEnum.NOT] = mappedFilter,
                        _b) : null;
            case ComparisonQueryOperatorEnum.EQUALS:
            case ComparisonQueryOperatorEnum.NOT_EQUALS:
                if (!filter.field)
                    throw new Error(QueryBuilderErrors.MISSING_FIELD);
                if (!filter.value && filter.value !== null)
                    throw new Error(QueryBuilderErrors.MISSING_VALUE);
                // if array is empty we will invalidate this query
                if (Array.isArray(filter.value) && filter.value.length === 0)
                    return null;
                return _c = {},
                    _c[filter.field] = (_d = {},
                        _d[filter.operator] = filter.value,
                        _d),
                    _c;
            case ComparisonQueryOperatorEnum.GREATER_THAN:
            case ComparisonQueryOperatorEnum.GREATER_THAN_EQUALS:
            case ComparisonQueryOperatorEnum.LESS_THAN:
            case ComparisonQueryOperatorEnum.LESS_THAN_EQUALS:
                if (!filter.field)
                    throw new Error(QueryBuilderErrors.MISSING_FIELD);
                if (!filter.value)
                    throw new Error(QueryBuilderErrors.MISSING_VALUE);
                return _e = {},
                    _e[filter.field] = (_f = {},
                        _f[filter.operator] = filter.value,
                        _f),
                    _e;
            case EvaluationQueryOperatorEnum.REGEX:
                if (!filter.field)
                    throw new Error(QueryBuilderErrors.MISSING_FIELD);
                if (!filter.value)
                    throw new Error(QueryBuilderErrors.MISSING_VALUE);
                // throw an error if not a string
                if (typeof filter.value !== 'string')
                    throw new Error(QueryBuilderErrors.NOT_A_STRING);
                // throw error if regex is invalid
                if (!(0, utils_1.isValidRegex)(filter.value))
                    throw new Error(QueryBuilderErrors.INVALID_REGEX);
                return _g = {},
                    _g[filter.field] = (_h = {},
                        _h[filter.operator] = filter.value,
                        _h),
                    _g;
            case ComparisonQueryOperatorEnum.IN:
            case ComparisonQueryOperatorEnum.NOT_IN:
            case ArrayQueryOperatorEnum.ALL:
                if (!filter.field)
                    throw new Error(QueryBuilderErrors.MISSING_FIELD);
                if (!filter.value)
                    throw new Error(QueryBuilderErrors.MISSING_VALUE);
                if (!Array.isArray(filter.value))
                    throw new Error(QueryBuilderErrors.NOT_AN_ARRAY);
                return _j = {},
                    _j[filter.field] = (_k = {},
                        _k[filter.operator] = filter.value,
                        _k),
                    _j;
            default:
                throw new Error(QueryBuilderErrors.UNKNOWN_OPERATOR);
        }
    };
    return QueryBuilder;
}());
exports.QueryBuilder = QueryBuilder;
//# sourceMappingURL=QueryBuilder.js.map